/**
 * Copyright (c) 2017-present, Stanislav Doskalenko - doskalenko.s@gmail.com
 * All rights reserved.
 *
 * This source code is licensed under the MIT-style license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * Based on Asim Malik android source code, copyright (c) 2015
 *
 **/
package com.reactnative.googlefit;

import static java.text.DateFormat.getDateInstance;
import static java.text.DateFormat.getTimeInstance;
import android.os.Looper;
import android.os.Handler;
import android.app.Activity;
import android.app.Dialog;
import android.content.Intent;
import android.content.IntentSender;
import android.content.DialogInterface;
import android.os.Bundle;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import android.util.Log;
import java.util.Date;
import java.util.Calendar;
import java.util.concurrent.TimeUnit;
import com.google.android.gms.tasks.Task;
import com.google.android.gms.tasks.Tasks;
import java.text.SimpleDateFormat;
import java.text.Format;
import java.util.concurrent.ExecutionException;
import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.bridge.WritableArray;
import com.facebook.react.bridge.WritableMap;

import com.facebook.react.bridge.ActivityEventListener;
import com.facebook.react.bridge.Callback;
import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.ReactContext;
import com.facebook.react.bridge.WritableMap;
import com.google.android.gms.fitness.ConfigClient;
import com.google.android.gms.fitness.FitnessOptions;
import com.google.android.gms.auth.api.signin.GoogleSignIn;
import com.google.android.gms.auth.api.signin.GoogleSignInClient;
import com.google.android.gms.auth.api.signin.GoogleSignInAccount;
import com.google.android.gms.auth.api.signin.GoogleSignInOptions;
import com.google.android.gms.common.api.GoogleApiClient;

import com.google.android.gms.fitness.HistoryClient;
import com.google.android.gms.fitness.data.Bucket;
import com.google.android.gms.fitness.data.DataPoint;
import com.google.android.gms.fitness.data.DataSet;
import com.google.android.gms.fitness.data.DataSource;
import com.google.android.gms.fitness.data.DataType;
import com.google.android.gms.fitness.data.Field;
import com.google.android.gms.fitness.Fitness;
import com.google.android.gms.fitness.result.DataReadResponse;
import com.google.android.gms.fitness.request.DataReadRequest;
import com.google.android.gms.tasks.OnFailureListener;
import com.google.android.gms.tasks.OnSuccessListener;
import java.util.List;
import java.text.DateFormat;
import java.lang.InterruptedException;
import java.lang.NullPointerException;

public class GoogleFitManager implements ActivityEventListener {
private ReactContext mReactContext;
private Activity mActivity;
private static final String TAG = "GoogleFitManager";
private static final int REQUEST_OAUTH_REQUEST_CODE = 1;
private Callback signInCallback;
private GoogleApiClient mGoogleApiClient;

public GoogleFitManager(ReactContext reactContext, Activity activity) {
        this.mReactContext = reactContext;
        this.mActivity      = activity;
        mReactContext.addActivityEventListener(this);
}


/** Gets {@link FitnessOptions} in order to check or request OAuth permission for the user. */
private FitnessOptions getFitnessSignInOptions() {
        return FitnessOptions.builder()
               .addDataType(DataType.TYPE_STEP_COUNT_DELTA, FitnessOptions.ACCESS_READ)
               .addDataType(DataType.AGGREGATE_STEP_COUNT_DELTA, FitnessOptions.ACCESS_READ)
               .addDataType(DataType.TYPE_DISTANCE_DELTA, FitnessOptions.ACCESS_READ)
               .addDataType(DataType.AGGREGATE_DISTANCE_DELTA, FitnessOptions.ACCESS_READ)
               .addDataType(DataType.TYPE_MOVE_MINUTES, FitnessOptions.ACCESS_READ)
               .addDataType(DataType.AGGREGATE_MOVE_MINUTES, FitnessOptions.ACCESS_READ)
               .build();
}


public void disconnect() {
        GoogleSignInAccount gsa = getLastSignedInAccountSafely();
        if (gsa != null) {
                try {
                  Tasks.await(Fitness.getConfigClient(mReactContext, gsa).disableFit());

                } catch (ExecutionException e) {
                        e.printStackTrace();
                } catch (InterruptedException e) {
                        e.printStackTrace();
                }
        }


}

public void authorize(@Nullable final Callback errorCallback, @Nullable final Callback successCallback) {
        final FitnessOptions fitnessOptions = getFitnessSignInOptions();
        signInCallback = successCallback;

        disconnect();

        GoogleSignIn.requestPermissions(
                mActivity,
                REQUEST_OAUTH_REQUEST_CODE,
                GoogleSignIn.getLastSignedInAccount(mActivity.getApplicationContext()),
                fitnessOptions);

}

private GoogleSignInAccount getLastSignedInAccountSafely() {
        GoogleSignInAccount account = GoogleSignIn.getLastSignedInAccount(mReactContext);
        if (account == null) {
                return null;
        }
        return account;
}


public WritableArray getActivities(long startTime, long endTime) {
        DateFormat dateFormat = DateFormat.getDateInstance();
        Log.i(TAG, "Range Start: " + dateFormat.format(startTime));
        Log.i(TAG, "Range End: " + dateFormat.format(endTime));

        DataReadRequest readRequest = new DataReadRequest.Builder()
                                      .aggregate(DataType.TYPE_STEP_COUNT_DELTA, DataType.AGGREGATE_STEP_COUNT_DELTA)
                                      .aggregate(DataType.TYPE_DISTANCE_DELTA, DataType.AGGREGATE_DISTANCE_DELTA)
                                      .aggregate(DataType.TYPE_MOVE_MINUTES, DataType.AGGREGATE_MOVE_MINUTES)
                                      .bucketByTime(1, TimeUnit.DAYS)
                                      .setTimeRange(startTime, endTime, TimeUnit.MILLISECONDS)
                                      .build();



        Task<DataReadResponse> response = Fitness.getHistoryClient(mActivity, GoogleSignIn.getLastSignedInAccount(mReactContext))
                                          .readData(readRequest);

        WritableArray activities = Arguments.createArray();
        DataReadResponse dataReadResponse;
        try {
                dataReadResponse =  Tasks.await(response);
                Log.i(TAG, "Number of buckets: " + dataReadResponse.getBuckets().size());
                for (Bucket bucket : dataReadResponse.getBuckets()) {
                        List<DataSet> dataSets = bucket.getDataSets();
                        processDataSet(dataSets, activities);
                }

        } catch (ExecutionException e) {
                e.printStackTrace();
        } catch (InterruptedException e) {
                e.printStackTrace();
        }


        return activities;
}

private void processDataSet(List<DataSet> dataSets, WritableArray activities) {
        WritableMap activityMap = Arguments.createMap();
        for (DataSet dataSet : dataSets) {
                Log.i(TAG, "Data returned for Data type: " + dataSet.getDataType().getName());
                DateFormat dateFormat = DateFormat.getDateInstance();
                DateFormat timeFormat = DateFormat.getTimeInstance();
                Format formatter = new SimpleDateFormat("EEE");
                for (DataPoint dp : dataSet.getDataPoints()) {
                        Log.i(TAG, "Data point:");
                        Log.i(TAG, "\tType: " + dp.getDataType().getName());
                 
                        Log.i(TAG, "\tStart: " + dateFormat.format(dp.getStartTime(TimeUnit.MILLISECONDS)));
                        Log.i(TAG, "\tEnd: " + dateFormat.format(dp.getEndTime(TimeUnit.MILLISECONDS)));

                        activityMap.putDouble("startDate", dp.getStartTime(TimeUnit.MILLISECONDS));
                        activityMap.putDouble("endDate", dp.getEndTime(TimeUnit.MILLISECONDS));
                        for (Field field : dp.getDataType().getFields()) {
                                Log.i(TAG, "\tValue: " + dp.getValue(field));
                                String fieldName = field.getName();
                                if (fieldName.equals("duration")) {
                                        activityMap.putDouble("minutes", dp.getValue(field).asInt());
                                }
                                if (fieldName.equals("distance")) {
                                        activityMap.putDouble("distance", dp.getValue(field).asFloat());
                                }
                                if (fieldName.equals("steps")) {
                                        activityMap.putDouble("steps", dp.getValue(field).asInt());
                                }
                        }
                }
        }
        activities.pushMap(activityMap);
}


public boolean isAuthorize() {
        FitnessOptions fitnessOptions = getFitnessSignInOptions();
        return GoogleSignIn.hasPermissions(GoogleSignIn.getLastSignedInAccount(mReactContext), fitnessOptions);
}

@Override
public void onNewIntent(Intent intent) {

}

@Override
public void onActivityResult(Activity activity, int requestCode, int resultCode, Intent data) {
        if (requestCode == REQUEST_OAUTH_REQUEST_CODE) {
                Log.i(TAG, "REQUEST CODE");
                if (signInCallback != null && resultCode == Activity.RESULT_OK) {
                        Log.i(TAG, "Authorization - Connected");
                        WritableMap map = Arguments.createMap();
                        map.putBoolean("authorized", true);
                        signInCallback.invoke(map);
                }
        }

}


}
