'use strict'

import { DeviceEventEmitter, NativeModules } from 'react-native'

const googleFit = NativeModules.RNGoogleFit

const METERS_PER_MILE = 0.000621371

class RNGoogleFit {
  constructor() {}

  authorize() {
    return new Promise((resolve, reject) => {
      googleFit.authorize((error) => resolve(error), (response) => resolve(response.authorized))
    })
  }

  authorizeWithSignIn() {
    return new Promise((resolve, reject) => {
      googleFit.authorizeWithSignIn((error) => resolve(error), (response) => resolve(response.authorized))
    })
  }

  /**
   * Get the total distance per day over a specified date range.
   * @param {Object} options getDailyDistanceSamples accepts an options object containing required startDate: ISO8601Timestamp and endDate: ISO8601Timestamp.
   * @callback callback The function will be called with an array of elements.
   */

  getActivities(options) {
    let startDate = Date.parse(options.startDate)
    let endDate = Date.parse(options.endDate)

    return new Promise((resolve, reject) => {
      googleFit.getActivities(
        startDate,
        endDate,
        (error) => {
          resolve(error)
        },
        (response) => {
          if (response.length > 0 && !!response[0].startDate) {
            const rawResponse = response[0]

            const distance = rawResponse.distance * METERS_PER_MILE
            const activities = {
              ...rawResponse,
              distance,
            }
            return resolve(activities)
          }

          return resolve({})
        },
      )
    })
  }

  isAvailable(callback) {
    // true if GoogleFit installed
    return new Promise((resolve, reject) => {
      googleFit.isAvailable(
        (error) => {
          resolve(error)
        },
        (response) => {
          resolve(response)
        },
      )
    })
  }

  isEnabled(callback) {
    // true if permission granted
    return new Promise((resolve, reject) => {
      googleFit.isEnabled(
        (error) => {
          resolve(error)
        },
        (response) => {
          resolve(response)
        },
      )
    })
  }
}

export default new RNGoogleFit()
